@@ -1,4 +1,4 @@
-# Copyright 2008 Canonical Ltd.
+# Copyright 2008,2012 Canonical Ltd.
 
 # This file is part of lazr.restfulclient.
 #
@@ -29,19 +29,21 @@ __all__ = [
     'RestfulHttp',
     ]
 
-
 import atexit
-import gzip
+import errno
+import os
 import shutil
+import sys
 import tempfile
 # Import sleep directly into the module so we can monkey-patch it
 # during a test.
 from time import sleep
 from httplib2 import (
-    FailedToDecompressContent, FileCache, Http, urlnorm)
+    Http,
+    urlnorm,
+    )
 import simplejson
 from cStringIO import StringIO
-import zlib
 
 from urllib import urlencode
 from wadllib.application import Application
@@ -49,6 +51,7 @@ from lazr.uri import URI
 from errors import error_for, HTTPError
 from _json import DatetimeJSONEncoder
 
+
 # A drop-in replacement for httplib2's safename.
 from httplib2 import _md5, re_url_scheme, re_slash
 def safename(filename):
@@ -136,7 +139,99 @@ class RestfulHttp(Http):
         return None
 
 
-class MultipleRepresentationCache(FileCache):
+class AtomicFileCache(object):
+    """A FileCache that can be shared by multiple processes.
+
+    Based on a patch found at
+    <http://code.google.com/p/httplib2/issues/detail?id=125>.
+    """
+
+    TEMPFILE_PREFIX = ".temp"
+
+    def __init__(self, cache, safe=safename):
+        """Construct an ``AtomicFileCache``.
+
+        :param cache: The directory to use as a cache.
+        :param safe: A function that takes a key and returns a name that's
+            safe to use as a filename.  The key must never return a string
+            that begins with ``TEMPFILE_PREFIX``.  By default uses
+            ``safename``.
+        """
+        self._cache_dir = os.path.normpath(cache)
+        self._get_safe_name = safe
+        try:
+             os.makedirs(self._cache_dir)
+        except OSError, e:
+            if e.errno != errno.EEXIST:
+                raise
+
+    def _get_key_path(self, key):
+        """Return the path on disk where ``key`` is stored."""
+        safe_key = self._get_safe_name(key)
+        if safe_key.startswith(self.TEMPFILE_PREFIX):
+            # If the cache key starts with the tempfile prefix, then it's
+            # possible that it will clash with a temporary file that we
+            # create.
+            raise ValueError(
+                "Cache key cannot start with '%s'" % self.TEMPFILE_PREFIX)
+        return os.path.join(self._cache_dir, safe_key)
+
+    def get(self, key):
+        """Get the value of ``key`` if set.
+
+        This behaves slightly differently to ``FileCache`` in that if
+        ``set()`` fails to store a key, this ``get()`` will behave as if that
+        key were never set whereas ``FileCache`` returns the empty string.
+
+        :param key: The key to retrieve.  Must be either bytes or unicode
+            text.
+        :return: The value of ``key`` if set, None otherwise.
+        """
+        cache_full_path = self._get_key_path(key)
+        try:
+            f = open(cache_full_path, 'rb')
+            try:
+                return f.read()
+            finally:
+                f.close()
+        except (IOError, OSError), e:
+            if e.errno != errno.ENOENT:
+                raise
+
+    def set(self, key, value):
+        """Set ``key`` to ``value``.
+
+        :param key: The key to set.  Must be either bytes or unicode text.
+        :param value: The value to set ``key`` to.  Must be bytes.
+        """
+        # Open a temporary file
+        handle, path_name = tempfile.mkstemp(
+            prefix=self.TEMPFILE_PREFIX, dir=self._cache_dir)
+        f = os.fdopen(handle, 'wb')
+        f.write(value)
+        f.close()
+        cache_full_path = self._get_key_path(key)
+        # And rename atomically (on POSIX at least)
+        if sys.platform == 'win32' and os.path.exists(cache_full_path):
+            os.unlink(cache_full_path)
+        os.rename(path_name, cache_full_path)
+
+    def delete(self, key):
+        """Delete ``key`` from the cache.
+
+        If ``key`` has not already been set then has no effect.
+
+        :param key: The key to delete.  Must be either bytes or unicode text.
+        """
+        cache_full_path = self._get_key_path(key)
+        try:
+            os.remove(cache_full_path)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+
+class MultipleRepresentationCache(AtomicFileCache):
     """A cache that can hold different representations of the same resource.
 
     If a resource has two representations with two media types,
