@@ -50,6 +50,7 @@ from lazr.restfulclient import __version__
 
 missing = object()
 
+
 class HeaderDictionary:
     """A dictionary that bridges httplib2's and wadllib's expectations.
 
@@ -218,8 +219,8 @@ class Resource(RestfulBase):
                 param_name + suffix)
             if param is not None:
                 try:
-                    value = param.get_value()
-                except KeyError, e:
+                    param.get_value()
+                except KeyError:
                     # The parameter could have been present, but isn't.
                     # Try the next parameter.
                     continue
@@ -242,7 +243,7 @@ class Resource(RestfulBase):
         :return: A NamedOperation instance that can be called with
                  appropriate arguments to invoke the operation.
         """
-        params = { 'ws.op' : operation_name }
+        params = {'ws.op': operation_name}
         method = self._wadl_resource.get_method('get', query_params=params)
         if method is None:
             method = self._wadl_resource.get_method(
@@ -369,7 +370,8 @@ class Resource(RestfulBase):
                 type_link = representation['resource_type_link']
                 if (type_link is not None
                     and type_link != self._wadl_resource.type_url):
-                    resource_type = self._root._wadl.get_resource_type(type_link)
+                    resource_type = self._root._wadl.get_resource_type(
+                        type_link)
                     self._wadl_resource.tag = resource_type.tag
             self.__dict__['_wadl_resource'] = self._wadl_resource.bind(
                 representation, self.JSON_MEDIA_TYPE,
@@ -388,7 +390,7 @@ class ScalarValue(Resource):
         """Return the scalar value."""
         self._ensure_representation()
         return self._wadl_resource.representation
-    
+
 
 class HostedFile(Resource):
     """A resource representing a file managed by a lazr.restful service."""
@@ -501,7 +503,7 @@ class ServiceRoot(Resource):
             # it with the service root resource.
             if url[:1] == '/':
                 url = url[1:]
-            url = self._root_uri.append(url)
+            url = str(self._root_uri.append(url))
         document = self._browser.get(url)
         try:
             representation = simplejson.loads(unicode(document))
@@ -568,9 +570,16 @@ class NamedOperation(RestfulBase):
             (media_type,
              in_representation) = self.wadl_method.build_representation(
                 **args)
-            extra_headers = { 'Content-type' : media_type }
+            extra_headers = {'Content-type': media_type}
+        # Pass uppercase method names to httplib2, as that is what it works
+        # with. If you pass a lowercase method name to httplib then it doesn't
+        # consider it to be a GET, PUT, etc., and so will do things like not
+        # cache. Wadl Methods return their method lower cased, which is how it
+        # is compared in this method, but httplib2 expects the opposite, hence
+        # the .upper() call.
         response, content = self.root._browser._request(
-            url, in_representation, http_method, extra_headers=extra_headers)
+            url, in_representation, http_method.upper(),
+            extra_headers=extra_headers)
 
         if response.status == 201:
             return self._handle_201_response(url, response, content)
@@ -799,7 +808,7 @@ class Collection(Resource):
             return self._get_slice(key)
         else:
             # Look up a single item by its position in the list.
-            found_slice = self._get_slice(slice(key, key+1))
+            found_slice = self._get_slice(slice(key, key + 1))
             if len(found_slice) != 1:
                 raise IndexError("list index out of range")
             return found_slice[0]
@@ -851,7 +860,7 @@ class Collection(Resource):
             page_url = self._with_url_query_variable_set(
                 self._wadl_resource.url, 'ws.start', start)
 
-        desired_size = stop-start
+        desired_size = stop - start
         more_needed = desired_size - len(entry_dicts)
 
         # Iterate over pages until we have the correct number of entries.
@@ -947,7 +956,10 @@ class CollectionWithKeyBasedLookup(Collection):
         try:
             shim_resource._ensure_representation()
         except HTTPError, e:
-            raise KeyError(key)
+            if e.response.status == 404:
+                raise KeyError(key)
+            else:
+                raise
         return shim_resource
 
     def __call__(self, key):
@@ -1050,5 +1062,5 @@ class HostedFileBuffer(StringIO):
             disposition = 'attachment; filename="%s"' % self.filename
             self.hosted_file._root._browser.put(
                 self.url, self.getvalue(),
-                self.content_type, {'Content-Disposition' : disposition})
+                self.content_type, {'Content-Disposition': disposition})
         StringIO.close(self)
